# Based off of howCode youtube video series "Make your own DNS servier in Python"
# https://www.youtube.com/playlist?list=PLBOh8f9FoHHhvO5e5HF_6mYvtZegobYX2
# https://github.com/howCodeORG/howDNS  Source code I started with, then butchered
#
# Also used https://pypi.org/project/pydnserver/ as a guide for the socket programming and timeouts
#  that project uses the dnspython library 
# https://stackoverflow.com/questions/17681230/how-make-dns-queries-in-dns-python-as-dig-with-additional-records-section
#
# details of the DNS Packet Structure
# http://www.tcpipguide.com/free/t_DNSMessageHeaderandQuestionSectionFormat.htm
#
# commands I found to see what process is using a port
#   sudo lsof -i :80
#   sudo lsof -i :53
#   sudo kill -9 pid 
#
# this was tested with dig
# sudo apt-get install dnsutils   (to get dig and nslookup)
# dig @192.168.1.200 microsoft.com
# dig @192.168.1.200 -p 53593 microsoft.com


import socket, logging, json
from datetime import datetime

# so I can forward DNS queries I cant answer
# Note that  gethostbyname_ex() looks in /etc/hosts for answeres before going out on the internet 
from socket import gethostbyname_ex     

dnsAbort = False
port = 53
#port = 53593   # port I used for testing with dig

# 'dvr_hosts' is a dictionary of 'hosts' for the PiGS DNS Server
# This is a list of all DNS lookups performed by the DVR+
# epg and roproxy are the guide servers
# DVR+ uses DNS requests to microsoft.com,amazon.com,google.com and maybe echostar.com to determine
#   if it is connected to the internet.  If the DNS query succeeds, DVR+ displays 'Connected' on the Network page
# tr50 redirects to stbfw.echodata.tv cname and is for firmware updates
# PiGS DNS forwards urls not its 'hosts' list to googledns 8.8.8.8 and returns that result

# this hosts dictionary is url IP pairs
#   '*' means IP of PiGS, which will be filled in by this module

# hosts now comes from pigs.json, this is the format
#dvr_hosts = {
#    'epg.channelmastertv.com' : '*',        # fw 1.35 guide requests go here
#    'roproxy-guide.echostarcdn.com' : '*',  # fw 1.34 and before guide requests go here
#    'pool.ntp.org' : '162.159.200.123',     # time update server requests
#    'echostar.com'  :  '69.164.198.192',    # DVR+ uses these next 4 to determine if it is connected to the Internet
#    'microsoft.com' : '173.255.215.209',    # These are just some ntp server IPs addresses
#    'amazon.com'    : '104.236.116.147',    # DVR+ doesnt like it if I return IP's on the same network
#    'google.com'    : '138.236.128.112',    # DVR+ doesnt access these addresses, it just wants DNS to succeed.
#    'tr50.dishaccess.tv': '127.0.0.1',      # A tr50.dishaccess.tv CNAME stbfw.echodata.tv A 67.148.153.225
#    'stbfw.echodata.tv' : '127.0.0.1'       # firmware update
#}

def getflags(flags):

    byte1 = bytes(flags[:1])
    byte2 = bytes(flags[1:2])

    rflags = '' # return flags string of binary 
    QR = '1'   # Set the Response bit
    OPCODE = ''
    for bit in range(1,5):
        OPCODE += str(ord(byte1)&(1<<bit))
    AA = '1'
    TC = '0'
    RD = '0'
    # Byte 2
    RA = '0'
    Z = '000'
    RCODE = '0000'

    # make binary strings, the convert to numbers, base 2
    bl = [int(QR+OPCODE+AA+TC+RD, 2), int(RA+Z+RCODE, 2)]
    ba = bytearray(bl)
    bx = int(QR+OPCODE+AA+TC+RD, 2).to_bytes(1, byteorder='big')+int(RA+Z+RCODE, 2).to_bytes(1, byteorder='big')
    return ba

def getquestiondomain(data):

    state = 0
    expectedlength = 0
    domainstring = ''
    domainparts = []
    x = 0
    y = 0
    for byte in data:
        if state == 1:
            if byte != 0:
                domainstring += chr(byte)
            x += 1
            if x == expectedlength:
                domainparts.append(domainstring)
                domainstring = ''
                state = 0
                x = 0
            if byte == 0:
                domainparts.append(domainstring)
                break
        else:
            state = 1
            expectedlength = byte
        y += 1

    questiontype = data[y:y+2]

    return (domainparts, questiontype)


def buildquestion(domainname, rectype):
    qbytes = b''

    for part in domainname:
        length = len(part)
        qbytes += bytes([length])

        for char in part:
            qbytes += ord(char).to_bytes(1, byteorder='big')

    if rectype == 'a':
        qbytes += (1).to_bytes(2, byteorder='big')

    qbytes += (1).to_bytes(2, byteorder='big')

    return qbytes

def rectobytes(domainname, rectype, recttl, recval):

    rbytes = b'\xc0\x0c'

    if rectype == 'a':
        rbytes = rbytes + bytes([0]) + bytes([1])

    rbytes = rbytes + bytes([0]) + bytes([1])

    rbytes += int(recttl).to_bytes(4, byteorder='big')

    if rectype == 'a':
        rbytes = rbytes + bytes([0]) + bytes([4])

        for part in recval.split('.'):
            rbytes += bytes([int(part)])
    return rbytes



def buildresponse(data, pigsIP, addr, hosts):

    # Get Transaction ID
    TransactionID = data[:2]

    # Get the flags, set response.  RCODE is the lowest 4-bits so we can just add it in
    Flags = getflags(data[2:4])

    # Question Count
    QDCOUNT = b'\x00\x01'

    # extract requested domain, check my hosts file
    domainname, questiontype = getquestiondomain(data[12:])
    
    qhost = '.'.join(domainname)[0:-1]  # strip off trailing '.'
    qhost = qhost.lower()
    
    logging.debug("PIGSDNS: DNS request from %s for %s", addr, qhost )
    
    # Create DNS body
    dnsbody = b''
    rectype = 'a'
    
    # Get answer for query, Check PiGS domains first, return PiGS IP if found
    if qhost in hosts :
        # if found, there is 1 record to return
        ANCOUNT = (1).to_bytes(2, byteorder='big')
        hostip  = hosts[qhost]
        if hostip == '*' :   # '*' means use IP discovered by PiGS
            hostip = pigsIP
        logging.debug("PIGSDNS: %s resolved to %s", qhost, hostip )
        dnsbody = rectobytes(domainname, rectype, 400, hostip)
    else:
        acnt = 0
        try :
            # forward anything else to the system DNS, use socket.gethostbyname_ex()
            hn, aliaslist, ipaddrlist = gethostbyname_ex(qhost)
            acnt = len(ipaddrlist)
            for ip in ipaddrlist:  # copy the ip answers to our dns response
                dnsbody += rectobytes(domainname, rectype, 400, ip )
            hostip = ipaddrlist[0]  # for debug
        except Exception as inst:
            #Flags[1] += 2   # signal a dns server error for all errors
            Flags[1] += 3   # signal a dns name error for all errors
            logging.warning("PIGSDNS:GHBN(%s) %s: %s", qhost, addr, inst )
        
        ANCOUNT = (acnt).to_bytes(2, byteorder='big')

    #logging.info("PIGSDNS: From %s %s resolved to %s", addr, qhost, hostip )

    # Nameserver, Additonal Count
    NSCOUNT = (0).to_bytes(2, byteorder='big')
    ARCOUNT = (0).to_bytes(2, byteorder='big')

    dnsheader = TransactionID+Flags+QDCOUNT+ANCOUNT+NSCOUNT+ARCOUNT

    dnsquestion = buildquestion(domainname, rectype)

    return dnsheader + dnsquestion + dnsbody


import threading

PigsDnsThread   = threading.Thread()

def PigsDnsStart(listenIP, IPaddr, hostsd) :
    global dnsAbort
    global PigsDnsThread
    
    PigsDnsThread = threading.Thread(target=PigsDNSserverRun, name='DNS-Server', daemon=True, args=(listenIP, IPaddr, hostsd,) )
    PigsDnsThread.start()

    logging.info("PIGSDNS: DnsStart ListenIP=%s, DefaultIP=%s, Alive:%s", listenIP, IPaddr, PigsDnsThread.is_alive())

    return PigsDnsThread

def PigsDnsStop() :
    global dnsAbort
    global PigsDnsThread
    dnsAbort = True
    return PigsDnsThread

def PigsDnsAlive() :
    global PigsDnsThread    
    return PigsDnsThread.is_alive()

def PigsDnsStatus() :
    global lastDnsRequestTime
    global PigsDnsThread    
    return PigsDnsThread.is_alive(), lastDnsRequestTime

# This is the 'run' or start method
requestCounter = 0
#lastDnsRequestTime = datetime.utcnow()
lastDnsRequestTime = datetime(2020, 1, 1)  # default to 00:00:00

def PigsDNSserverRun( listen_ip, pigsIP, dnshosts) :
    global dnsAbort
    global requestCounter
    global lastDnsRequestTime
    
    timeoutSecs = 1
    timeoutCounter = 0
    
    dnsAbort = False 
    
    # set up the dns server will return to DVR
    respIP = pigsIP
    if listen_ip != '0.0.0.0' :  # if dns is going to listen to a specific IP, that will be the default response IP
        respIP = listen_ip
    
    logging.info("PIGSDNS: Default IP %s", respIP )

    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.settimeout(timeoutSecs)  # recvfrom will timeout every 1 second, ensures we can abort thread
    
    try :
        sock.bind((listen_ip, port))
        logging.info("PIGSDNS: Waiting for DNS requests on port %s", port )
    except:
        logging.warning("PIGSDNS: Could not bind to port %s", port )
        logging.warning("PIGSDNS: dnsmasq loaded?  try 'sudo lsof -i :53' for pid" )
        dnsAbort = True

    while not dnsAbort:
        try :
            data, addr = sock.recvfrom(512)  # DNS UDP messages limited to 512 bytes
            requestCounter += 1
            lastDnsRequestTime = datetime.utcnow()  # time of this request

            r = buildresponse(data, respIP, addr, dnshosts)
            sock.sendto(r, addr)
        except (socket.timeout, socket.error):  # periodic timeout is expected
            
            timeoutCounter += 1   # count timeouts, warn every 5 mins if DVR's not making DNS requests
            if timeoutCounter >= 300/timeoutSecs :
                #logging.info("PIGSDNS: Requests: %d  Time: %s...", requestCounter, timeoutCounter )
                if requestCounter == 0 :
                    logging.warning("PIGSDNS: Has not received a DVR DNS request for %s seconds...", timeoutCounter*timeoutSecs )
                
                requestCounter = 0;
                timeoutCounter = 0
            continue

        except Exception as err:
            logging.error(u'PIGSDNS: DNS Server main thread err: {err}'.format(err=err))

    logging.info("PIGSDNS: Stopping DNS & closing socket.")
    
    try :
        sock.shutdown(socket.SHUT_RDWR)
    except :
        pass
    try :
        sock.close()
    except :
        pass
